<?php

namespace OrganizationsManagement\Controller;

use Exception;
use OrganizationsManagement\Traits\TQuery;
use OrganizationsManagement\Traits\TResponse;
use OrganizationsManagement\Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource;
use OrganizationsManagement\Service;
use OrganizationsManagement\Util\InstanceProvider;
use OrganizationsManagement\Service\VersionRegistration as VersionRegistrationService;

class VersionRegistration extends Simple
{

    use TResponse;

    /**
     * @var VersionRegistrationService
     */
    private $versionRegistrationService;

    /**
     * VersionRegistration constructor.
     * @param VersionRegistrationService
     */
    public function __construct(VersionRegistrationService $versionRegistrationService)
    {
        $this->versionRegistrationService = $versionRegistrationService;
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function findVersionRegistration(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $this->versionRegistrationService->findVersionRegistration();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function saveVersionRegistration(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $request->getParameters();
            $this->versionRegistrationService->updateCell($data);
            $this->sendResponse(['success' => 'Registro(s) atualizado(s) com sucesso.'], 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
