import { Metadata, Http, IEventParam } from '@zeedhi/core';
import { Modal, ModalService, DialogService, Select } from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { OMInstance } from '@/services/OMInstance';
import Messages from '@/services/Messages';

Metadata.get('metadata/downloadVersionDelphi/modals/addClientVersionModal.json');
Metadata.get('metadata/downloadVersionDelphi/modals/deleteVersionModal.json');

export class DownloadVersionDelphiController extends DialogService {
  private includeModal!: Modal;

  private deleteModal!: Modal;

  public gridApi: any;

  public selectedClient: any;

  private masterCurrentRow: any = {};

  public clienteName = '';

  public hasMasterCurrentRow = false;

  public loadingButtons = false;

  public disableButtons = false;

  public async rowClick() {
    await this.createAddClientVersionModal();
    this.openAddClientVersionModal();
  }

  public async createAddClientVersionModal() {
    if (!this.includeModal) {
        this.includeModal = await ModalService.createFromJson('downloadVersionDelphi/modals/addClientVersionModal', true);
    }
  }

  public openAddClientVersionModal() {
    this.includeModal.show();
  }

  public closeAddClientVersionModal() {
    this.includeModal.hide();
  }

  public gridReadyEvent({ api }: any) {
    this.gridApi = api;
    return this.gridApi;
  }

  public selectedRows({ data, node }: any) {
    if (!node.selected) {
      return;
    }
    this.gridApi.getSelectedNodes();
    this.selectedClient = data;
    this.selectedRows([]);
  }

  public async onRowClickedClienteGrid(params: any) {
    this.clienteName = params.row.NMFANTCLIE;
    const { datasource } = OMInstance.getInstanceTekGrid('versionGrid');
    datasource.filter.CDCLIENTE = params.row.CDCLIENTE;
    this.controllerProperty();
    await datasource.get();
  }

  get clienteNameSpan() {
    return `<span style="white-space: nowrap"><span style='font-size: 20px; font-weight: 700;'>Cliente: </span><span style="font-size: 17px">${this.clienteName}</span></span>`;
  }

  public controllerProperty() {
    this.hasMasterCurrentRow = true;
  }

  public selectedCheckboxRows({ event }: IEventParam<TekGrid>) {
    event?.preventDefault();
    this.showMessageDeleteAllVersions();
  }

  public static showDeleteMessage() {
    const msg = Messages.SUCCESS_DELETE_DEAFULT;
    this.show({
      name: 'success_dialog',
      type: 'success',
      title: 'SUCESSO',
      text: msg,
    });
  }

  public static showSucessMessage() {
    const msg = Messages.SUCCESS_SAVE_DEAFULT;
    this.show({
      name: 'success_dialog',
      type: 'success',
      title: 'SUCESSO',
      text: msg,
    });
  }

  public static showErrorMessage() {
    const msg = Messages.ERROR_DEFAULT;
    this.show({
      name: 'error_dialog',
      type: 'error',
      title: 'Erro',
      text: msg,
    });
  }

  private back: any = {
    click: () => DialogService.hide(),
  };

  private deletarAllVersions: any = {
    click: () => this.excluirTodasAsVersoes(),
  };

  public showMessageDeleteAllVersions() {
   DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'Confirmação da operação',
      text: 'Deseja excluir todas as versões selecionadas?',
      buttons: [
        { name: 'yes', component: 'ZdButton', label: 'YES', events: this.deletarAllVersions, outline: true },
        { name: 'no', component: 'ZdButton', label: 'NO', events: this.back },
      ],
    });
  }

  public async excluirTodasAsVersoes() {
    const grid = Metadata.getInstance<TekGrid>('clienteGrid');
    const gridVersion = Metadata.getInstance<TekGrid>('versionGrid');
    const uri = '/deleteAllVersion';
    const rows: string[] = [];
    const val = grid.datasource.data.map((row: any) => {
      const isSelected = grid.selectedRows.some((selectedRow) => selectedRow.CDCLIENTE === row.CDCLIENTE);
      if (isSelected) {
          const editedRow = { ...row, LAST_VERSION: '' };
          rows.push(editedRow.CDCLIENTE);
          return editedRow;
        }
        return row;
    });
    try {
        await Http.delete(uri, { params: { cdcliente: rows } });
        DownloadVersionDelphiController.showDeleteMessage();
    } catch {
        DownloadVersionDelphiController.showErrorMessage();
    }
    grid.datasource.updateData(val);
    grid.selectedRows = [];
    grid.reload();
    gridVersion.reload();
  }

  public async incluirVersao() {
    this.loadingButtons = true;
    this.disableButtons = true;
    const grid = Metadata.getInstance<TekGrid>('clienteGrid');
    const gridVersion = Metadata.getInstance<TekGrid>('versionGrid');
    const uri = '/incluirVersoes';
    const selectedGrid = Metadata.getInstance<Select>('clientSelect');
    const versionGrid = Metadata.getInstance<Select>('versionSelect');
    let cdclientesRows: string[] = [];
    const val = versionGrid.value.map((item) => item.NRVERSAODISP);
    cdclientesRows = selectedGrid.value;
    try {
        await Http.post(uri, { nrversaodisps: val, cdclientes: cdclientesRows });
        DownloadVersionDelphiController.showSucessMessage();
    } catch {
        DownloadVersionDelphiController.showErrorMessage();
    }
    grid.reload();
    gridVersion.reload();
    this.closeAddClientVersionModal();
    this.loadingButtons = false;
    this.disableButtons = false;
  }

  public async deleteVersao() {
   this.loadingButtons = true;
   this.disableButtons = true;
   DialogService.hide();
   const grid = Metadata.getInstance<TekGrid>('clienteGrid');
   const gridVersion = Metadata.getInstance<TekGrid>('versionGrid');
   const uri = '/deleteVersoes';
   const selectedGrid = Metadata.getInstance<Select>('deleteVersionClientSelect');
   const versionGrid = Metadata.getInstance<Select>('deleteVersionSelect');
   let cdclientesRows: string[] = [];
   const val = versionGrid.value.map((item) => item.NRVERSAODISP);
   cdclientesRows = selectedGrid.value;
   try {
       await Http.delete(uri, { params: { nrversaodisps: val, cdclientes: cdclientesRows } });
       DownloadVersionDelphiController.showDeleteMessage();
   } catch {
       DownloadVersionDelphiController.showErrorMessage();
   }
   grid.reload();
   gridVersion.reload();
   this.closeDeleteModal();
   this.loadingButtons = false;
   this.disableButtons = false;
 }

  private deletarVersions: any = {
    click: () => this.deleteVersao(),
  };

  public showMessageDeleteAllVersionsWithClients() {
    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'Confirmação da operação',
      text: 'Deseja excluir a(s) versão(ões)?',
      buttons: [
        { name: 'yes', component: 'ZdButton', label: 'YES', events: this.deletarVersions, outline: true },
        { name: 'no', component: 'ZdButton', label: 'NO', events: this.back },
      ],
    });
  }

  public catchVersions() {
    const versionGrid = Metadata.getInstance<Select>('deleteVersionSelect');
    const versions = versionGrid.value.map((item) => item.VERSION);
    return versions;
  }

  public async incluirVersaoForAllClients() {
    this.loadingButtons = true;
    this.disableButtons = true;
    const grid = Metadata.getInstance<TekGrid>('clienteGrid');
    const gridVersion = Metadata.getInstance<TekGrid>('versionGrid');
    const uri = '/incluirParaTodosVersoes';
    const versionGrid = Metadata.getInstance<Select>('versionSelect');
    const val = versionGrid.value.map((item) => item.NRVERSAODISP);
    try {
        await Http.post(uri, { nrversaodisps: val });
        DownloadVersionDelphiController.showSucessMessage();
    } catch {
        DownloadVersionDelphiController.showErrorMessage();
    }
    grid.reload();
    gridVersion.reload();
    this.closeAddClientVersionModal();
    this.loadingButtons = false;
    this.disableButtons = false;
  }

  public async deleteTodasVersoes() {
    this.loadingButtons = true;
    this.disableButtons = true;
    DialogService.hide();
    const grid = Metadata.getInstance<TekGrid>('clienteGrid');
    const gridVersion = Metadata.getInstance<TekGrid>('versionGrid');
    const uri = '/deleteParaTodosVersoes';
    const versionGrid = Metadata.getInstance<Select>('deleteVersionSelect');
    const val = versionGrid.value.map((item) => item.NRVERSAODISP);
    try {
        await Http.delete(uri, { params: { nrversaodisps: val } });
        DownloadVersionDelphiController.showSucessMessage();
    } catch {
        DownloadVersionDelphiController.showErrorMessage();
    }
    grid.reload();
    gridVersion.reload();
    this.closeDeleteModal();
    this.loadingButtons = false;
    this.disableButtons = false;
  }

  private deletarTodasVersions: any = {
    click: () => this.deleteTodasVersoes(),
  };

  public showMessageDeleteAllVersionsWithoutChooseClients() {
    const versions: string[] = this.catchVersions();
    const versionsCut = versions.join(', ');
    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'Confirmação da operação',
      text: `Deseja excluir a(s) versão(ões) ${versionsCut} de todos os clientes?`,
      buttons: [
        { name: 'yes', component: 'ZdButton', label: 'YES', events: this.deletarTodasVersions, outline: true },
        { name: 'no', component: 'ZdButton', label: 'NO', events: this.back },
      ],
    });
  }

  public async rowClickDeleteVersion() {
    await this.createDeleteModal();
    this.openDeleteModal();
  }

  public async createDeleteModal() {
    if (!this.deleteModal) {
     this.deleteModal = await ModalService.createFromJson('downloadVersionDelphi/modals/deleteVersionModal', true);
    }
  }

  public openDeleteModal() {
    this.deleteModal.show();
  }

  public closeDeleteModal() {
    this.deleteModal.hide();
  }
}
